/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "Transmission/Generated/SDK_Service.h"
#include "Transmission/Generated/SharedTextureCommunication.h"
#include "IMixCast.h"
#include "Tracking/SdkCustomTrackedObjectMediator.h"
#include "Transmission/Thrift/ServiceSDKHandler.h"
#include "Transmission/Thrift/ThriftClient.h"
#include "Transmission/Thrift/ThriftServer.h"

#include "Engine/TextureRenderTarget2D.h"
#include "Containers/Queue.h"
#include "HAL/Runnable.h"
#include "HAL/RunnableThread.h"

#include <vector>
#include <memory>

using namespace mixcast::data;
using namespace mixcast::thrift;

class FServiceMutexThread;

// Manages RPC communication to the MixCast Client
class MIXCAST_API FServiceCommunicator
{
	friend class FServiceMutexThread;
	friend class AMixCastSdkActor;
	friend class SdkCustomTrackedObjectMediator;
public:
	FServiceCommunicator();
	~FServiceCommunicator();

	bool SendExperienceEvent(const FString& EventId) const;
	bool SetAutoSnapshot(bool bEnable, const FString& CameraId) const;
	bool SetVideoRecording(bool bEnable, const FString& CameraId) const;
	bool SetVideoStreaming(bool bEnable, const FString& CameraId) const;
	bool TakeSnapshot(const FString& CameraId) const;

	SharedTex GetSharedTextureInfo(const FString& TextureId);

	void UpdateCustomTrackedObjects() { CustomTrackedObjects.UpdateClientIfChanged(this); }
private:
	typedef TPair<FString, const UTextureRenderTarget2D*> SharedTextureSource;

	FDelegateHandle ActiveChangedDelegate;

	//Receiving
	std::shared_ptr<FServiceSDKHandler> ServiceSDKHandler;
	TThriftServer<Service_SDKProcessor, Service_SDKIf> ServiceSDKServer;

	//Sending
	int missedPingCount;
	TThriftClient<SDK_ServiceClient> SDKServiceClient;
	TThriftClient<SharedTextureCommunicationClient> SharedTextureClient;
	TQueue<SharedTextureSource, EQueueMode::Spsc> SharedTextureQueue;
	int SharedTextureQueueSize = 0;

	FRunnable* ServiceMutexCheck;

	SdkCustomTrackedObjectMediator CustomTrackedObjects;

	void NotifySdkStarted() const;
	void NotifySdkStopped() const;

	void NotifySdkChanged() const;

	void UpdateExperienceTrackedObjects(const std::vector<TrackedObject>& objs);

	void HandleActiveChanged();

	static ExperienceMetadata GetExpData();
};
