#pragma once

#include "OwnedUnrealSharedTexture_D3D12.h"
#include "WatchedUnrealSharedTexture.h"
#include "../../../MixCastLog.h"

#if MIXCAST_D3D12

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif
#include <d3d12.h>
#include "ThirdParty/Windows/D3DX12/Include/d3dx12.h"
#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif

#include "ITextureShareD3D12.h"

OwnedUnrealSharedTexture_D3D12::OwnedUnrealSharedTexture_D3D12(UTexture* srcTex)
{
	this->srcTex = srcTex;
	ID3D12Resource* src = (ID3D12Resource*)GetNativeTexture(*srcTex);
	src->GetDevice(IID_PPV_ARGS(&device));

	D3D12_RESOURCE_DESC srcDesc = src->GetDesc();

	D3D12_RESOURCE_DESC dstDesc = {};
	dstDesc.MipLevels = 1;
	dstDesc.Format = EnsureShareableFormat(srcDesc.Format);
	dstDesc.Width = srcDesc.Width;
	dstDesc.Height = srcDesc.Height;
	dstDesc.Flags = D3D12_RESOURCE_FLAG_ALLOW_RENDER_TARGET | D3D12_RESOURCE_FLAG_ALLOW_SIMULTANEOUS_ACCESS;
	dstDesc.DepthOrArraySize = 1;
	dstDesc.SampleDesc.Count = 1;
	dstDesc.SampleDesc.Quality = 0;
	dstDesc.Dimension = D3D12_RESOURCE_DIMENSION_TEXTURE2D;

	try
	{
		const D3D12_HEAP_PROPERTIES props = CD3DX12_HEAP_PROPERTIES(D3D12_HEAP_TYPE_DEFAULT);
		HRESULT hr = device->CreateCommittedResource(
			&props,
			D3D12_HEAP_FLAG_SHARED,
			&dstDesc,
			D3D12_RESOURCE_STATE_COMMON | D3D12_RESOURCE_STATE_COPY_DEST,
			nullptr,
			IID_PPV_ARGS(&sharedTex));

		if (FAILED(hr)) {
			return;
		}
	}
	catch (...)
	{
		MIXCAST_ERROR("Failed to create shared texture");
		return;
	}

	try
	{
		HRESULT hr = device->CreateSharedHandle(sharedTex, NULL, GENERIC_ALL, NULL, &sharedHandle);

		if (FAILED(hr)) {
			return;
		}
	}
	catch (...)
	{
		SAFE_RELEASE(sharedTex);
		sharedHandle = NULL;

		MIXCAST_ERROR("Failed to get shared handle");
		return;
	}

	if (!ITextureShareD3D12::Get().CreateRHITexture(sharedTex, WatchedUnrealSharedTexture::GetUnrealFormatFromDirectXFormat(dstDesc.Format), sharedTexRhi))
		MIXCAST_ERROR("Couldn't create RHI for shared owned texture");
	sharedTexRhi->SetName(FName("Shared Tex"));
}
OwnedUnrealSharedTexture_D3D12::~OwnedUnrealSharedTexture_D3D12()
{
	if (sharedHandle == 0)
		return;

	SAFE_RELEASE(sharedTexRhi);

	SAFE_RELEASE(sharedTex);
	CloseHandle(sharedHandle);
}

void OwnedUnrealSharedTexture_D3D12::Write_RenderThread(FRHICommandListImmediate& RHICmdList)
{
	if (srcTex == NULL || sharedTex == NULL || !srcTex->IsValidLowLevel())
		return;

	FBox2D CopyBox(FVector2D(0.0f, 0.0f), FVector2D(srcTex->GetSurfaceWidth(), srcTex->GetSurfaceHeight()));
	GDynamicRHI->RHICopySubTextureRegion(srcTex->Resource->GetTexture2DRHI(), sharedTexRhi, CopyBox, CopyBox);
}
#endif