/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"
#include "Engine/Engine.h"
#include "UObject/WeakObjectPtrTemplates.h"
#include "Components/SceneComponent.h"

#include "Transmission/Generated/TrackedObject_types.h"

#include "CustomTrackedObjectComponent.generated.h"

using namespace mixcast::data;

UENUM()
namespace EObjectType
{
	enum Type
	{
		Unknown = 0,
		Hmd,
		Controller,
		Tracker,
		BaseStation,
		Virtual,
	};
}


UENUM()
namespace EAssignedRole
{
	enum Type
	{
		Unknown = 0,
		Head,
		LeftHand,
		RightHand,
	};
}

UCLASS(HideCategories = ("Activation", "Actor", "Collision", "Cooking", "HLOD", "Input", "LOD", "Physics", "Rendering", "Replication", "Tags", "Asset User Data"),
	ClassGroup=(MixCast), meta=(BlueprintSpawnableComponent) )
class MIXCAST_API UCustomTrackedObjectComponent : public USceneComponent
{
	GENERATED_BODY()

public:	
	UCustomTrackedObjectComponent();

protected:
	virtual void BeginPlay() override;
	virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;

public:	
	static TArray<UCustomTrackedObjectComponent*> Active;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "MixCast Info",
		meta = (Tooltip = "This should be a unique identifier for the tracked object, which can be referenced on successive application runs"))
		FString ObjectIdentifier = "";

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "MixCast Info",
		meta = (Tooltip = "This should be a human-readable name for the tracked object"))
		FString ObjectName = "";

	UPROPERTY(EditAnywhere, BlueprintReadWrite, AdvancedDisplay, Category = "MixCast Info",
		meta = (Tooltip = "This describes the form factor of the tracked object, which may be used in determining if/how MixCast uses it"))
		TEnumAsByte<EObjectType::Type> ObjectType = EObjectType::Virtual;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, AdvancedDisplay, Category = "MixCast Info",
		meta = (Tooltip = "Should only be set if the head or hands aren't already specified through OpenVR or Oculus"))
		TEnumAsByte<EAssignedRole::Type> ObjectRole = EAssignedRole::Unknown;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, AdvancedDisplay, Category = "MixCast Info",
		meta = (Tooltip = "Should be false unless you are creating tracked objects that will be programmatically accessed"))
		bool HideFromUser = false;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, AdvancedDisplay, Category = "MixCast Info",
		meta = (Tooltip = "If this is set, poses reported to MixCast will be relative to the origin's root component transform rather than being in world space"))
		AActor* Origin;

	FVector GetSharedPosition() const;
	FQuat GetSharedRotation() const;


	static ObjectType::type UnrealObjectTypeToThrift(const EObjectType::Type& role)
	{
		switch (role)
		{
		case EObjectType::Unknown:
			return ObjectType::UNKNOWN;
		case EObjectType::Hmd:
			return ObjectType::HMD;
		case EObjectType::Controller:
			return ObjectType::CONTROLLER;
		case EObjectType::Tracker:
			return ObjectType::TRACKER;
		case EObjectType::BaseStation:
			return ObjectType::BASE_STATION;
		case EObjectType::Virtual:
			return ObjectType::VIRTUAL;
		}
		return ObjectType::UNKNOWN;
	}

	static AssignedRole::type UnrealAssignedRoleToThrift(const EAssignedRole::Type& role)
	{
		switch (role)
		{
		case EAssignedRole::Unknown:
			return AssignedRole::UNKNOWN;
		case EAssignedRole::Head:
			return AssignedRole::HEAD;
		case EAssignedRole::LeftHand:
			return AssignedRole::LEFT_HAND;
		case EAssignedRole::RightHand:
			return AssignedRole::RIGHT_HAND;
		}
		return AssignedRole::UNKNOWN;
	}
};
